var App = function(){
    'use strict';
    
    /////////////////////////////////////////////
    ///
    /// HELLO STRANGER.
    /// WHAT ARE YOU DOING HERE?
    ///
    /////////////////////////////////////////////

    var self = this,
        PROJECT_NUMBER = '442663907604',
        BACKEND_URI = 'http://pro.merckgroup.com/c168ca55d9be78aa4d419169c3b5a8f991568600',
        //BACKEND_URI = 'http://merck-dev.sopg.net/c168ca55d9be78aa4d419169c3b5a8f991568600',
        REDIRECT_TO_PAGE = 'http://pro.merckgroup.com/',
        URL_PARAM = '?ios=true',
        WINDOW_TITLE = 'Merck Pro',
        AUTH_USER = 'merck',
        AUTH_PASS = 'merck$$';
    
    /**
     * initialize instance properties
     */
    this.ready = false;
    this.PushNotification = null;
    this.platform = null;
    this.locale = null;
    this.fallbackLocale = 'en';
    this.list = document.getElementById('msglist');
    this.offlineDefinition = ['unknown', 'none'];


    // store for translations
    this.translations = {
        'de': {
            'offlineMessage': 'Sie sind leider zur Zeit offline'
        },
        'en': {
            'offlineMessage': 'Unfortunately you are currently offline'
        },
        'fr': {
            'offlineMessage': 'Malheureusement, vous n\'êtes pas en ligne actuellement'
        },
        'es': {
            'offlineMessage': 'Lo sentimos, en estos momentos estás desconectado'
        },
        'ja': {
            'offlineMessage': '残念ながら、あなたは現在オフラインです'
        },
        'zh':{
            'offlineMessage': '很抱歉，当前为离线状态'
        },
        'pt':{
            'offlineMessage': 'Infelizmente, você está offline no momento'
        },
        'it':{
            'offlineMessage': 'Purtroppo, in questo momento Lei è offline'
        },
        'ko':{
            'offlineMessage': '안타깝지만, 귀하는 현재 오프라인 상태입니다'
        },
        'ru': {
            'offlineMessage': 'Нет связи с интернетом'
        }
    };

    /**
     * register the device ready callback
     */
    this.initialize = function(){
        document.addEventListener('deviceready', self.onDeviceReady, false);

        //self.onDeviceReady();
    };
    
    /**
     * fired when Phonegap plugins are loaded and the device is ready.
     */
    this.onDeviceReady = function(){

        // somehow this evt is firing twice Oo
        if( self.ready ){
            return;
        }

        self.ready = true;

        self.log('device ready.');
        self.showLoader();
        // initialize instance properties
        self.platform = window.device.platform;

        self.log('setting device locale');

        // fetch language, then
        self.setPreferredLanguage(function(){

            // true if valid connection
            if( self.checkConnection() ){
                
                self.log('initializing PushNotification');
                // instantiate module(s)
                self.PushNotification = new PushNotification();

                self.log('registering pushNotifications');
                // register push notification handlers
                self.registerPushNotifications(function(){
                    self.redirectToPage();
                });

            }else{
                // show offline message
                self.log('offline message.');
                self.hideLoader();
                self.showOfflineMessage();
            }
        });
    };

    /**
     * returns true of a valid connection is available
     * @return {bool}
     */
    this.checkConnection = function() {
        if( self.offlineDefinition.indexOf( navigator.connection.type ) === -1 ){
            return true;
        }
        return false;
    };
    
    /**
     * sets the devices locale
     */
    this.setPreferredLanguage = function( cb ){
        window.navigator.globalization.getPreferredLanguage(
            function( lang ){
                var lng = lang.value.split('-');
                self.locale = lng[0];
                self.log('device locale: ' + lang.value);
                cb();
            },
            function(){
                self.log('no device locale.');
                cb();
            }
        );
    };
    
    /**
     * registers device to push notification Service
     * either GCM or APNS
     * @param {function} cb 
     */
    this.registerPushNotifications = function( cb ){
        if( 'Android' === self.platform || 'android' === self.platform  ){
            self.PushNotification.register(
                self.pushTokenHandler.bind(self, cb),
                self.pushError.bind(self, cb),
                {
                    'senderID': PROJECT_NUMBER,
                    'ecb': 'MERCK.onNotification'
                }
            );
        }else{
            self.PushNotification.register(
                self.pushTokenHandler.bind(self, cb),
                self.pushError.bind(self, cb),
                {
                    "badge": true,
                    "sound": true,
                    "alert": true,
                    'ecb': 'MERCK.onNotificationAPN'
                }
            );
        }
        self.log('registered notifications');
    };
    
    /**
     * recieves the push token id.
     * we want this id forwarded to the application server.
     * @param  {number} result
     */
    this.pushTokenHandler = function( cb, result ){
        self.log( 'ios token: ' + result );
        
        // for some reason
        if( result !== "OK" ){
            self.forwardClientId( 'ios', result, cb );
        }
        
    };
    
    /**
     * @global callback for APNS Notification
     * @param  {object} event
     */
    this.onNotificationAPN = function( event ){
        self.log('onNotificationAPN cb');
        
        if( event.alert ){
            self.showNotificationAlert( event.alert );
        }
        
        if( event.sound ){
            var snd = new Media(event.sound);
            snd.play();
        }
        
        if( event.bdge ){
            pushNotification.setApplicationIconBadgeNumber(
               function(){ alert('Badge Success'); },
               function(){ alert('Badge Error'); },
               0
            );
        }
    };
    
    /**
     * handles a GCM Connection Callback
     * @param  {object} e
     */
    this.onNotification = function( e ){
        
        self.log( 'onNotification. event: ' + e.event );
        
        switch( e.event ){
            case 'registered':
                self.log( 'registered:' + e.regid );
                self.forwardClientId( 'android', e.regid, self.redirectToPage );
                break;
                
            case 'message':
                self.log( 'message content:' + e.payload.message );
                self.showNotificationAlert(e.payload.message);
                // handle message if App is open
                break;
                
            case 'error':
                // handle error somehow
                break;
                
            default:
                // unknown message
                break;
        }
    };

    /**
     * shows a pushnotification message
     * @return void
     */
    this.showNotificationAlert = function( message ){
        navigator.notification.alert( message, function(){
            // do something when dialog is closed?
            // nop. not yet.
        }, WINDOW_TITLE, 'OK');
    };
    
    /**
     * LETS DO SOME VANILLA JS.
     * WE DONT NEED JQUIZZ
     * @param  {string} platform
     * @param  {number|string} id
     */
    this.forwardClientId = function( platform, id, cb ){
        
        var xhr = new XMLHttpRequest();
        xhr.open('POST', BACKEND_URI, true, AUTH_USER, AUTH_PASS);
        xhr.onreadystatechange = function(){
            if( xhr.readyState != 4 || xhr.status != 200 ){
                self.log( 'state: ' + xhr.readyState + "; status: " + xhr.status );
            }else{
                cb();
                self.log( 'success: ' + xhr.responseText );
            }
        };
        xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
        self.log('sending XHR...');
        self.log("clientId=" + encodeURIComponent(id) + "&platform=" + encodeURIComponent(platform) + '&locale=' + encodeURIComponent(self.locale) );
        xhr.send("clientId=" + encodeURIComponent(id) + "&platform=" + encodeURIComponent(platform) + '&locale=' + encodeURIComponent(self.locale) );
    };
    
    /**
     * push notification error callback
     * @param  {mixed} result
     */
    this.pushError = function( cb, error ){
        self.log( 'error' + error );
        cb();
    };

    /**
     * redirects the 'browser' to the given Page
     * @return void
     */
    this.redirectToPage = function(){
        //self.hideLoader();
    };
    
    /**
     * shows the offline Message.
     */
    this.showOfflineMessage = function(){
        var el = document.getElementById('offline-message'),
            message = self.__( 'offlineMessage' );

        el.innerHTML = message;
    };

    /**
     * shows the loading animation
     * @return void
     */
    this.showLoader = function(){
        var el = document.getElementById('loader-container');
        el.style.display = "block";
    };

    /**
     * hides the loading animation
     * @return void
     */
    this.hideLoader = function(){
        var el = document.getElementById('loader-container');
        el.style.display = "none";
    };

    /**
     * our main translation function.
     * @param  {string} key
     * @return {string|number}
     */
    this.__ = function( key ){
        var node = self.translations[ self.locale ];

        if( node && node[ key ] ){
            return node[ key ];
        }else if(
            self.translations[ self.fallbackLocale ]
            &&
            self.translations[ self.fallbackLocale ][ key ]
        ){
            return self.translations[ self.fallbackLocale ][ key ];
        }else{
            self.log('translation not found for ' + key);
        }
    };

    /**
     * @debug appends a msg to a list
     * @param  {string} result
     */
    this.log = function( msg ){
        var li = document.createElement('li');
        li.appendChild(document.createTextNode( msg ));
        //return self.list.appendChild(li);
        console.log(msg);
    };
    
    this.initialize();
};

// init app
window.MERCK = new App();